<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayer\Generator;

use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Flat101\EnhancedEcommerce\Model\DataLayer\Dto;
use Flat101\EnhancedEcommerce\Model\DataLayer\Generator;
use Magento\Checkout\Model\Session;
use Magento\Sales\Model\Order\Item;
use Magento\Store\Model\StoreManagerInterface;

class CheckoutStep
{
    use UnsetProperty;

    private const EVENT = 'checkoutStep';
    private const EVENT_CATEGORY = 'enhanced ecommerce';
    private const EVENT_ACTION = 'checkout';

    /** @var Session */
    private $session;

    /** @var StoreManagerInterface */
    private $storeManager;

    /** @var Generator\Product */
    private $productGenerator;

    public function __construct(
        Session $session,
        StoreManagerInterface $storeManager,
        Generator\Product $productGenerator
    ) {
        $this->session = $session;
        $this->storeManager = $storeManager;
        $this->productGenerator = $productGenerator;
    }

    public function onCheckoutStep(
        int $step,
        $option,
        $eventLabel = null,
        $orderPlaced = false
    ): Dto\Ecommerce
    {
        $products = [];

        // When order is already placed
        if ($orderPlaced) {
            $order = $this->session->getLastRealOrder();

            /** @var Item $item */
            foreach ($order->getAllVisibleItems() as $item) {
                $products[] = $this->productGenerator->generate($item->getProduct(), $item->getQtyOrdered(), $item);
            }
        }

        // When order is not placed
        if (!$orderPlaced) {
            foreach ($this->session->getQuote()->getAllVisibleItems() as $item) {
                $products[] = $this->productGenerator->generate($item->getProduct(), $item->getQty(), $item);
            }
        }

        $actionField = new Dto\Cart\ActionField();
        $actionField->step = $step;
        $actionField->option = $option;
        if (is_null($option)) {
            $this->unset($actionField, ['option']);
        }

        $detailCart = new Dto\EcommerceDetails();
        $detailCart->actionField = $actionField;
        $detailCart->products = $products;

        $checkout = new Dto\Cart\Checkout();
        $checkout->currencyCode = $this->storeManager->getStore()->getCurrentCurrency()->getCode();
        $checkout->checkout = $detailCart;

        $ecommerce = new Dto\Ecommerce();
        $ecommerce->event = self::EVENT;
        $ecommerce->eventCategory = self::EVENT_CATEGORY;
        $ecommerce->eventAction = self::EVENT_ACTION;
        $ecommerce->eventLabel = $eventLabel;
        $ecommerce->ecommerce = $checkout;

        if (is_null($eventLabel)) {
            $this->unset($ecommerce, ['eventLabel']);
        }
        return $ecommerce;
    }
}
