<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayerGa4\Builder;

use Flat101\EnhancedEcommerce\Model\DataLayerGa4\Dto;
use Flat101\EnhancedEcommerce\Model\Handler;
use Flat101\EnhancedEcommerce\Helper\Config;
use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Model\Product as ProductEntity;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Framework\Pricing\Helper\Data as PricingHelper;

class ImpressionBuilder
{
    use UnsetProperty;

    /** @var Handler\ProductHandler */
    private $productHandler;

    /** @var CategoryRepositoryInterface */
    private $categoryRepository;

    /** @var Config */
    private $config;

    /** @var Configurable */
    private $configurableProduct;

    /** @var PricingHelper */
    private $pricingHelper;

    public function __construct(
        Handler\ProductHandler $productHandler,
        CategoryRepositoryInterface $categoryRepository,
        Config $config,
        Configurable $configurableProduct,
        PricingHelper $pricingHelper
    ) {
        $this->productHandler = $productHandler;
        $this->categoryRepository = $categoryRepository;
        $this->config = $config;
        $this->configurableProduct = $configurableProduct;
        $this->pricingHelper = $pricingHelper;
    }

    public function build(
        ProductEntity $entity,
        string $list,
        array $additionalInfo
    ): Dto\Impression\ImpressionProduct {
        $this->productHandler->setProduct($entity);

        $productImpressionDto = new Dto\Impression\ImpressionProduct();
        $productImpressionDto->item_id
            = $entity->getData($this->productHandler->productIdentifier());
        $productImpressionDto->item_name = $entity->getName();
        $productImpressionDto->price = $this->getPrice($entity);

        // TODO: Check if correct category placed here
        $productImpressionDto->item_category = $this->productHandler->getCategoryName();

        $categoryPath = [];
        if (array_key_exists('item_category', $additionalInfo)) {
            $categoryPath[] = $additionalInfo['item_category'];
        }
        if (array_key_exists('item_category2', $additionalInfo)) {
            $categoryPath[] = $additionalInfo['item_category2'];
        }
        if (array_key_exists('item_category3', $additionalInfo)) {
            $categoryPath[] = $additionalInfo['item_category3'];
        }
        if (array_key_exists('item_category4', $additionalInfo)) {
            $categoryPath[] = $additionalInfo['item_category4'];
        }
        if (array_key_exists('item_category5', $additionalInfo)) {
            $categoryPath[] = $additionalInfo['item_category5'];
        }

        if (empty($categoryPath)) {
            $categoryPath = $this->productHandler->getCategoryPath(true, true);
        }
        $i = 1;
        foreach ($categoryPath as $categoryItem) {
            if ($i == 1) {
                $productImpressionDto->item_category = $categoryItem;
            }
            if ($i > 1) {
                $productImpressionDto->{'item_category' . $i} = $categoryItem;
            }
            $i++;
        }

        if ($brandValue = $this->productHandler->getBrandValue()) {
            $productImpressionDto->item_brand = $brandValue;
        }

        if (!is_null($variantValue = $this->checkVariantValue($entity))) {
            $productImpressionDto->item_variant = $variantValue;
        }

        $productImpressionDto->index = $additionalInfo['position'] ?? '0';

        if (!is_null($list)) {
            $productImpressionDto->item_list_name = $list;
        } else {
            $this->unset($productImpressionDto, ['item_list_name']);
        }

        if (!isset($productImpressionDto->item_category)) {
            $this->unset($productImpressionDto, ['item_category']);
        }
        if (!isset($productImpressionDto->item_category2)) {
            $this->unset($productImpressionDto, ['item_category2']);
        }
        if (!isset($productImpressionDto->item_category3)) {
            $this->unset($productImpressionDto, ['item_category3']);
        }
        if (!isset($productImpressionDto->item_category4)) {
            $this->unset($productImpressionDto, ['item_category4']);
        }
        if (!isset($productImpressionDto->item_category5)) {
            $this->unset($productImpressionDto, ['item_category5']);
        }

        return $productImpressionDto;
    }

    private function getPrice(ProductEntity $product): float
    {
        if ($product->getTypeId() === Configurable::TYPE_CODE) {
            return floatval($this->getConfigurablePrice($product));
        }
        return floatval($product->getSpecialPrice()) ?: floatval($product->getData('price'));
    }

    private function getConfigurablePrice(ProductEntity $product)
    {
        $childProductPrice = [];
        $childProducts = $this->configurableProduct->getUsedProducts($product);

        foreach ($childProducts as $child) {
            $price = number_format(floatval($child->getPrice()), 2, '.', '');
            $finalPrice = number_format(floatval($child->getFinalPrice()), 2,
                '.', '');

            if ($price == $finalPrice) {
                $childProductPrice[] = $price;
            } elseif ($finalPrice < $price) {
                $childProductPrice[] = $finalPrice;
            }
        }

        return min($childProductPrice);
    }

    private function checkVariantValue($item): ?string
    {
        return $this->productHandler->checkVariantValue($item);
    }
}
