<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayerGa4\Generator;

use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4\Dto;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4\Generator;
use Magento\Checkout\Model\Session;
use Magento\Sales\Model\Order\Item;
use Magento\Store\Model\StoreManagerInterface;

class CheckoutStep
{
    use UnsetProperty;

    private const EVENT_CATEGORY = 'enhanced ecommerce';
    private const EVENT_ACTION = 'checkout';

    /** @var Session */
    private $session;

    /** @var StoreManagerInterface */
    private $storeManager;

    /** @var Generator\Product */
    private $productGenerator;

    public function __construct(
        Session $session,
        StoreManagerInterface $storeManager,
        Generator\Product $productGenerator
    ) {
        $this->session = $session;
        $this->storeManager = $storeManager;
        $this->productGenerator = $productGenerator;
    }

    public function onCheckoutStep(
        $eventName,
        $orderPlaced = null
    ): Dto\Ecommerce
    {
        $products = [];
        $shippingTier = null;
        $paymentType = null;
        $coupon = null;
        $discount = null;

        // When order is already placed
        if ($orderPlaced) {
            /** @var Item $item */
            foreach ($orderPlaced->getAllVisibleItems() as $item) {
                $products[] = $this->productGenerator->generate($item->getProduct(), $item->getQtyOrdered(), $item);
            }
//            $shippingTier = $order->getShippingDescription();
            $paymentType = $orderPlaced->getPayment()->getMethodInstance()->getTitle();
            $coupon = $orderPlaced->getCouponCode() ?? null;
            $discount = $orderPlaced->getDiscountAmount() > 0 ? $orderPlaced->getDiscountAmount() : null;
        }

        // When order is not placed
        if (is_null($orderPlaced)) {
            foreach ($this->session->getQuote()->getAllVisibleItems() as $item) {
                $products[] = $this->productGenerator->generate($item->getProduct(), $item->getQty(), $item);
            }
            $shippingTier = $this->session->getQuote()->getShippingAddress()->getShippingDescription();
        }

        $detailCart = new Dto\EcommerceDetails();
        $detailCart->shipping_tier = $shippingTier;
        $detailCart->payment_type = $paymentType;
        $detailCart->coupon = $coupon;
        $detailCart->discount = $discount;
        $detailCart->currency = $this->storeManager->getStore()->getCurrentCurrency()->getCode();
        $detailCart->items = $products;

        if ($orderPlaced) {
            $this->unset($detailCart, ['shipping_tier']);

            if (null === $coupon) {
                $this->unset($detailCart, ['coupon']);
            }
            if (null === $discount) {
                $this->unset($detailCart, ['discount']);
            }
        }

        if (!$orderPlaced) {
            $this->unset($detailCart, ['payment_type','coupon','discount']);
        }

        $ecommerce = new Dto\Ecommerce();
        $ecommerce->event = $eventName;
        $ecommerce->ecommerce = $detailCart;

        return $ecommerce;
    }
}
